import requests
from enum import Enum

class http:
    resFile = "Helper/response_codes.txt"

    class methods(Enum):
        GET = 1
        HEAD = 2
        POST = 3
        PUT = 4
        DELETE = 5
        CONNECT = 6
        OPTIONS = 7
        TRACE = 8

    def sendRequest(
            method=methods.GET,
            url=None,
            data=None,
            json=None,
            params=None,
            headers=None,
            cookies=None,
            files=None,
            auth=None,
            timeout=None,
            allow_redirects=True,
            proxies=None,
            hooks=None,
            stream=None,
            verify=None,
            cert=None
    ):
        methods = http.methods

        if method == methods.TRACE or method == methods.CONNECT:
            raise NotImplemented
        else:
            r = getattr(requests, method.name.lower())(
                url=url, data=data
            )
        if (r.status_code != 200):
            print(f"{r.status_code} {http.findCode(r.status_code)}")
        return r


    def findCode(code):
        list = []
        f = open(http.resFile)
        list = f.readlines()

        for line in list:
            if line.startswith(f"{code}"):
                ans = line.split("=")[1]

        return ans


import json

from .emitter import Emitter
from .HTTPHelper import http


class Client:
    server = "https://cry-vs.herokuapp.com"
    funcs = []

    def listen(func):
        Client.funcs.append(func)

    def login(self, key, allowUnsecure = False):


        server = Client.server
        if server.lower().startswith("http://") and allowUnsecure == False:
            print(f"{server} could not be loaded as it is http. please change it to https or set alowUnsecure to True in the function paramaters.")
            return

        r = http.sendRequest (
            method=http.methods.POST,
            url=server + "/login",
            data=json.dumps({
                "key": key
            })
        )


        emitter = Emitter(funcs=self.funcs, expire=r.headers["Expire"])
        emitter.enqueue(name="on_ready")
        emitter.queue()



import queue

class Emitter:
    q = queue.Queue()

    def __init__(self,
                 funcs,
                 expire,
                 events=[
                     "any_event",
                     "on_ready"
                 ],
                 ):
        self.funcs = funcs
        self.events = events

    def queue(self):
        while True:
            if self.q.empty(): continue
            curr = self.q.get()
            for func in self.funcs:
                if func.__name__ in self.events and func.__name__ == curr[0].lower():
                    try:
                        func(*curr[1])
                    except TypeError:
                        func()

    def enqueue(self, name, args=[]):
        self.q.put(item=[name, args])


import subprocess

import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="cry-vs.py",
    version="0.0.0.0",
    author="addikted",
    # author_email="author@example.com",
    description="A Crypto-Versus wrapper",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/AW1534/cry-vs.py",
    project_urls={
        "Bug Tracker": "https://github.com/AW1534/cry-vs.py/issues",
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: GNU Affero General Public License v3",
        "Operating System :: OS Independent",
        "Development Status :: 2 - Pre-Alpha",
        "Natural Language :: English",
        "Topic :: Internet :: WWW/HTTP"
    ],
    package_dir={"": "src"},
    packages=setuptools.find_packages(where="src"),
    python_requires=">=3.6",
    install_requires=[
        "requests>=2.27.1"
    ]
)